@@ -1,26 +1,34 @@
 #
-# $Id: Split.pm,v 0.46 2004/01/06 12:31:57 sts Exp $
+# $Id: Split.pm,v 0.48 2004/01/07 17:00:37 sts Exp $
 
 package Dir::Split;
 
-our $VERSION = '0.46';
+our $VERSION = '0.48';
 
 use 5.6.1;
-use strict;
+use strict 'vars';
 use warnings;
 
+use File::Basename;
 use File::Copy 'cp';
 use File::Path;
 use File::Spec;
+use SelfLoader;
 
 our (# external data
         @exists,
         %failure,
         %track,
 
+        $traverse,
+        $traverse_unlink,
+        $traverse_rmdir,
+
+     # return
         $Ret_status,
 
      # data
+        @Dirs,
         @Files,
         %F_names_case,
         %F_names_char,
@@ -239,6 +247,7 @@ sub split {
         $o->_sort_files() if $o->{'mode'} eq 'num';
         $o->_suffix();
         $o->_move();
+        $o->_traversed_rm_dir() if ${__PACKAGE__.'::traverse'};
 
         $Ret_status = FAILURE if %failure;
     }
@@ -332,8 +341,13 @@ sub _sanity_input {
 sub _gather_files {
     my $o = $_[0];
 
-    $o->_dir_read($o->{'source'}, \@Files);
-    @Files = grep !-d File::Spec->catfile($o->{'source'}, $_), @Files;
+    if (${__PACKAGE__.'::traverse'}) {
+        $o->_traverse(\@Dirs, \@Files);
+    }
+    else {
+        $o->_dir_read($o->{'source'}, \@Files);
+        @Files = grep !-d File::Spec->catfile($o->{'source'}, $_), @Files;
+    }
 
     $track{'source'}{'files'} = scalar @Files;
 }
@@ -397,7 +411,7 @@ sub _suffix_num_contin {
         ($_) = /(.+?)$sep(.*)/;
         my $suff_cmp = $2;
         # increase suffix to highest number
-        if ($o->{'ident'} eq $_ && $suff_cmp =~ /[0-9]/) {
+        if ($o->{'ident'} eq $_ && $suff_cmp =~ /[0-9]/o) {
             $Suffix = $suff_cmp if $suff_cmp > $Suffix;
         }
     }
@@ -431,7 +445,8 @@ sub _suffix_char {
     my $o = $_[0];
 
     foreach my $file (@Files) {
-        $_ = $file;
+        if (${__PACKAGE__.'::traverse'}) { $_ = basename($file) }
+        else { $_ = $file }
         s/\s//g if /\s/; # whitespaces
         ($_) = /^(.{$o->{'length'}})/;
         if ($_ =~ /\w/) {
@@ -540,20 +555,30 @@ sub _mkpath {
 sub _cp_unlink {
     my ($o, $file) = @_;
 
-    my $path_full = File::Spec->catfile($Path, $file);
-    $file = File::Spec->catfile($o->{'source'}, $file);
+    my $path_target;
+    if (${__PACKAGE__.'::traverse'}) {
+        $path_target = File::Spec->catfile($Path, basename($file));
+    }
+    else {
+        $path_target = File::Spec->catfile($Path, $file);
+        $file = File::Spec->catfile($o->{'source'}, $file);
+    }
 
-    if ($o->_exists_and_not_override($path_full)) {
-        push @exists, $path_full;
+    if ($o->_exists_and_not_override($path_target)) {
+        push @exists, $path_target;
         return;
     }
 
-    unless (cp $file, $path_full) {
-        push @{$failure{'copy'}}, $path_full;
+    unless (cp $file, $path_target) {
+        push @{$failure{'copy'}}, $path_target;
         return;
     }
     $track{'target'}{'files'}++;
 
+    if (${__PACKAGE__.'::traverse'}) {
+        return unless ${__PACKAGE__.'::traverse_unlink'};
+    }
+
     unless (unlink $file) {
         push @{$failure{'unlink'}}, $file;
         return;
@@ -584,6 +609,7 @@ sub _exists_and_not_override {
 #
 
 sub _clean_up {
+    undef @Dirs;
     undef %F_names_case;
     undef %F_names_char;
     undef $Path;
@@ -591,6 +617,74 @@ sub _clean_up {
 }
 
 1;
+
+__DATA__
+
+#
+# _traverse (\@dirs, \@files)
+#
+# Traversal machinery.
+#
+
+sub _traverse {
+    my ($o, $dirs_ref, $files_ref) = @_;
+
+    my ($count, @dirs, $eval, @eval, @files,
+        $limit, @stack);
+    $count = 0;
+
+    $o->_dir_read($o->{'source'}, \@eval);
+    @eval =
+      map { File::Spec->catfile($o->{'source'}, $_) }
+      @eval;
+
+    RECURSE:
+    while (@eval) {
+        if (-d $eval[0]) {
+            $eval = shift @eval;
+            push @dirs, $eval;
+        }
+        elsif (-f $eval[0]) {
+            my $file = shift @eval;
+            push @files, $file;
+            next;
+        }
+        else { croak 'Traversing failed: unknown type' }
+
+        my @items;
+        $o->_dir_read($eval, \@items);
+
+        foreach (@items) {
+            my $item = File::Spec->catfile($eval, $_);
+            if (-d $item) { push @stack, $item }
+            else { push @files, $item }
+        }
+
+        unless (@eval) {
+            #if ($limit && ++$count == $limit) { last RECURSE }
+            @eval = @stack;
+            undef @stack;
+        }
+    }
+
+    @$dirs_ref  = @dirs;
+    @$files_ref = @files;
+}
+
+#
+# _traversed_rm_dir()
+#
+# Removes traversed directories.
+#
+
+sub _traversed_rm_dir {
+    if (${__PACKAGE__.'::traverse_rmdir'}) {
+        foreach (@Dirs) {
+            rmtree($_, 1, 1);
+        }
+    }
+}
+
 __END__
 
 =head1 OPTIONS
@@ -639,8 +733,6 @@ B<options> (mandatory)
 
 =over 4
 
-=over 4
-
 =item *
 
 =item B<mode>
@@ -714,8 +806,6 @@ of the highest suffix number.
 
 =back
 
-=back
-
 =head2 characteristic
 
 Split files to subdirectories with a characteristic suffix. Files
@@ -742,8 +832,6 @@ B<options> (mandatory)
 
 =over 4
 
-=over 4
-
 =item *
 
 =item B<mode>
@@ -805,8 +893,6 @@ Whitespaces in suffixes will be removed.
 
 =back
 
-=back
-
 =head2 tracking
 
 C<%Dir::Split::track> keeps count of how many files the source and directories / files
@@ -850,6 +936,25 @@ It is recommended to evaluate those arrays on FAILURE.
 
 A C<@Dir::Split::exists> array may coexist.
 
+=head2 traversing
+
+Traversal processing of files within the source directory may not be activated by passing
+an argument to the object constructor, it requires the following variable to be set to true:
+
+ # traversal mode
+ $Dir::Split::traverse        = 1;
+
+Additionally some options:
+
+ # unlink files in source
+ $Dir::Split::traverse_unlink = 1;
+
+ # remove directories in source
+ $Dir::Split::traverse_rmdir  = 1;
+
+It is B<NOT> recommended to turn on the latter options C<$Dir::Split::traverse_unlink> and
+C<$Dir::Split::traverse_rmdir>, unless you're aware of the consequences they imply.
+
 =head1 EXAMPLES
 
 Assuming F</source> contains 5 files:
@@ -863,6 +968,7 @@ Assuming F</source> contains 5 files:
 After splitting the directory tree in F</target> will look as following:
 
 B<numeric splitting>
+
  +- system-00001
  +-- _12.tmp
  +-- abc.tmp
@@ -913,11 +1019,6 @@ should be well grounded.
 
 =back
 
-=head1 LIMITATIONS
-
-Recursive source directory processing is not supported;
-existing directories within the source directory will silently be ignored.
-
 =head1 DEPENDENCIES
 
 C<Perl 5.6.1>; C<File::Copy>, C<File::Path>, C<File::Spec>.
